import 'package:easy_localization/easy_localization.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';

import 'package:scrobbles/cubit/data_timeline_cubit.dart';
import 'package:scrobbles/cubit/settings_cubit.dart';
import 'package:scrobbles/models/timeline.dart';
import 'package:scrobbles/network/scrobbles.dart';
import 'package:scrobbles/ui/widgets/card_content.dart';
import 'package:scrobbles/ui/widgets/charts/timeline_counts.dart';
import 'package:scrobbles/ui/widgets/charts/timeline_eclecticism.dart';
import 'package:scrobbles/ui/widgets/error.dart';

class CardTimeline extends StatelessWidget {
  const CardTimeline({super.key});

  @override
  Widget build(BuildContext context) {
    SettingsCubit settings = BlocProvider.of<SettingsCubit>(context);

    final int daysCount = settings.getTimelineDaysCount();

    return BlocBuilder<DataTimelineCubit, DataTimelineState>(
      builder: (BuildContext context, DataTimelineState data) {
        final TimelineData timeline = data.timeline ?? TimelineData.fromJson({});

        return CardContent(
          color: Theme.of(context).colorScheme.surface,
          title: 'timeline_title'.tr(
            namedArgs: {
              'daysCount': daysCount.toString(),
            },
          ),
          loader: update(daysCount),
          content: Stack(
            children: [
              ChartTimelineCounts(chartData: timeline),
              ChartTimelineEclecticism(chartData: timeline),
            ],
          ),
        );
      },
    );
  }

  Widget update(int daysCount) {
    const Widget loading = Text('⏳');
    const Widget done = Text('');

    late Future<TimelineData> future = ScrobblesApi.fetchTimeline(daysCount);

    return BlocBuilder<DataTimelineCubit, DataTimelineState>(
      builder: (BuildContext context, DataTimelineState data) {
        return FutureBuilder<TimelineData>(
          future: future,
          builder: (context, snapshot) {
            if (snapshot.hasError) {
              return ShowErrorWidget(message: '${snapshot.error}');
            }

            BlocProvider.of<DataTimelineCubit>(context).update(snapshot.data);

            return !snapshot.hasData ? loading : done;
          },
        );
      },
    );
  }
}
